#!/usr/bin/env python3

import math
import random
import argparse
import seaborn as sns


iTOL_usage = '''
==================================== iTOL example commands ====================================

# Example commands
BioSAK iTOL -ColorStrip -lg MagTaxon.txt -lt Phylum -out ColorStrip_taxon.txt
BioSAK iTOL -ColorRange -lg MagTaxon.txt -lt Phylum -out ColorRange_taxon.txt
BioSAK iTOL -SimpleBar -lv MagSize.txt -scale 0-3-6-9 -lt Size -out SimpleBar_size.txt
BioSAK iTOL -Heatmap -lm MagAbundance.txt -lt Abundance -out Heatmap_abundance.txt
BioSAK iTOL -ExternalShape -lm identity_matrix.txt -lt Identity -out ExternalShape_identity.txt -scale 25-50-75-100

# Leaf-to-Group file format (-lg, tab separated, no header)
genome_1	Bacteria
genome_2	Archaea

# Group-to-Color and Column-to-Color file format (-gc and -cc, tab separated, no header)
Bacteria	#CCCC00
Archaea	#9999FF

# Leaf-to-Value file format (-lv, tab separated, no header)
genome_1	6.15
genome_2	6.63

# Leaf-to-Matrix file format (-lm, tab separated, header required!!!)
Genome_id Sample_A   Sample_B   Sample_C
genome_1	6.15    2.23    1.56
genome_2	6.63    1.72    2.55

# Note!!!
The purpose for developing this module is to generate iTOL-recognizable file for your dataset, 
parameters (e.g. colours and font size) specified in the outputs might need further adjustments. 

===============================================================================================
'''


def get_color_list(color_num):

    if color_num <= 8:
        color_list_combined = ['#3787c0', '#39399f', '#ffb939', '#399f39', '#9f399f', '#fb694a', '#9f9f39', '#959595']

    elif 8 < color_num <= 16:
        color_list_combined = ['#2b7bba', '#89bedc', '#2e2e99', '#8a8acc', '#ffa500', '#ffc55c', '#2e992e', '#8acc8a', '#992e99', '#cc8acc', '#d52221', '#fc8161', '#99992e', '#cccc8a', '#5c5c5c', '#adadad']

    else:
        color_num_each = math.ceil(color_num/8) + 2

        color_list_1 = sns.color_palette('Blues',  n_colors=color_num_each).as_hex()
        color_list_2 = sns.light_palette('navy',   n_colors=color_num_each).as_hex()
        color_list_3 = sns.light_palette('orange', n_colors=color_num_each).as_hex()
        color_list_4 = sns.light_palette('green',  n_colors=color_num_each).as_hex()
        color_list_5 = sns.light_palette('purple', n_colors=color_num_each).as_hex()
        color_list_6 = sns.color_palette('Reds',   n_colors=color_num_each).as_hex()
        color_list_7 = sns.light_palette('olive',  n_colors=color_num_each).as_hex()
        color_list_8 = sns.color_palette('Greys', n_colors=color_num_each).as_hex()

        color_list_combined = []
        for color_list in [color_list_1, color_list_2, color_list_3, color_list_4, color_list_5, color_list_6, color_list_7, color_list_8]:
            for color in color_list[2:][::-1]:
                color_list_combined.append(color)

    color_list_to_return = random.sample(color_list_combined, color_num)

    color_list_to_return_sorted = []
    for color_to_return in color_list_combined:
        if color_to_return in color_list_to_return:
            color_list_to_return_sorted.append(color_to_return)

    return color_list_to_return_sorted


def scale_str_to_size_list(scale_str):

    scale_list = scale_str.split('-')
    scale_list = [float(i) for i in scale_list]

    shape_size_list = []
    if scale_list[0] == 0:
        shape_size_list = [0]
        for each_value in scale_list[1:-1]:
            current_size = each_value/scale_list[-1]
            shape_size_list.append(current_size)
        shape_size_list.append(1)

    if scale_list[0] != 0:
        shape_size_list = [0.1]
        interval_num = len(scale_list) - 1
        interval_value = (1 - 0.1)/interval_num
        n = 1
        for each_value in scale_list[1:-1]:
            shape_size_list.append(interval_value * n + 0.1)
            n += 1
        shape_size_list.append(1)

    return shape_size_list


def iTOL(args):

    # read in arguments
    ColorStrip      = args['ColorStrip']
    ColorRange      = args['ColorRange']
    SimpleBar       = args['SimpleBar']
    Heatmap         = args['Heatmap']
    ExternalShape   = args['ExternalShape']
    LeafGroup       = args['lg']
    GroupColor      = args['gc']
    ColumnColor     = args['cc']
    LeafValue       = args['lv']
    LeafMatrix      = args['lm']
    scale_str       = args['scale']
    LegendTitle     = args['lt']
    FileOut         = args['out']

    # General
    STRIP_WIDTH                 = 100
    MARGIN                      = 20

    # SimpleBar
    SimpleBar_COLOR             = 'grey'
    SimpleBar_WIDTH             = 300
    SimpleBar_HEIGHT_FACTOR     = 0.8
    SimpleBar_BORDER_WIDTH      = 0
    SimpleBar_SCALE_COLOR       = '#696969'
    SimpleBar_SCALE_WIDTH       = 1
    SimpleBar_SCALE_DASHED      = 1
    SimpleBar_SCALE_FontSize    = 2

    # Heatmap
    Heatmap_STRIP_WIDTH         = 60

    # ExternalShape

    # check the number of specified file type
    True_num = 0
    for file_type in [ColorStrip, ColorRange, SimpleBar, Heatmap, ExternalShape]:
        if file_type is True:
            True_num += 1

    if True_num == 0:
        print('Please specify one file type, choose from -ColorStrip, -ColorRange, -SimpleBar, -Heatmap or -ExternalShape')
        exit()
    if True_num > 1:
        print('Please specify one file type ONLY, choose from -ColorStrip, -ColorRange, -SimpleBar, -Heatmap or -ExternalShape')
        exit()

    ####################################################################################################################

    # Prepare ColorStrip and ColorRange file
    if (ColorStrip is True) or (ColorRange is True):

        Leaf_to_Group_dict = {}
        Group_list = []
        for each_leaf in open(LeafGroup):
            each_leaf_split = each_leaf.strip().split('\t')
            Leaf_to_Group_dict[each_leaf_split[0]] = each_leaf_split[1]
            # get Group_list
            if each_leaf_split[1] not in Group_list:
                Group_list.append(each_leaf_split[1])

        Group_to_Color_dict = {}
        if GroupColor is None:
            color_list = get_color_list(len(Group_list))
            Group_to_Color_dict = dict(zip(Group_list, color_list))
        else:
            # get groups with provided color
            Group_to_provided_Color_dict = dict()
            group_with_provided_color_list = []
            for each_group in open(GroupColor):
                each_group_split = each_group.strip().split('\t')
                Group_to_provided_Color_dict[each_group_split[0]] = each_group_split[1]
                group_with_provided_color_list.append(each_group_split[0])

            # assign colors to the rest groups
            group_without_color_list = []
            for each_group in Group_list:
                if each_group not in group_with_provided_color_list:
                    group_without_color_list.append(each_group)
            if len(group_without_color_list) > 0:
                color_list_unprovided = get_color_list(len(group_without_color_list))
                Group_to_Color_dict_unprovided = dict(zip(group_without_color_list, color_list_unprovided))
                for each_group in Group_to_Color_dict_unprovided:
                    Group_to_Color_dict[each_group] = Group_to_Color_dict_unprovided[each_group]

            # combine two dict
            for each_group in Group_to_provided_Color_dict:
                Group_to_Color_dict[each_group] = Group_to_provided_Color_dict[each_group]

        group_list = [i for i in Group_to_Color_dict]
        color_list = [Group_to_Color_dict[i] for i in group_list]

        FileOut_handle = open(FileOut, 'w')

        # write out header
        if ColorStrip is True:
            FileOut_handle.write('DATASET_COLORSTRIP\n')
            FileOut_handle.write('SEPARATOR TAB\n')
            FileOut_handle.write('DATASET_LABEL\t%s_ColorStrip\n' % LegendTitle)
        if ColorRange is True:
            FileOut_handle.write('TREE_COLORS\n')
            FileOut_handle.write('SEPARATOR TAB\n')
            FileOut_handle.write('DATASET_LABEL\t%s_ColorRange\n' % LegendTitle)

        # write out strip attributes
        if ColorStrip is True:
            FileOut_handle.write('\n# customize strip attributes here\n')
            FileOut_handle.write('STRIP_WIDTH\t%s\n' % STRIP_WIDTH)
            FileOut_handle.write('MARGIN\t%s\n'      % MARGIN)

        # write out legend info
        FileOut_handle.write('\n# customize legend here\n')
        FileOut_handle.write('LEGEND_TITLE\t%s\n' % LegendTitle)
        FileOut_handle.write('LEGEND_SHAPES\t%s\n' % '\t'.join(['1' for i in Group_to_Color_dict]))
        FileOut_handle.write('LEGEND_COLORS\t%s\n' % '\t'.join(color_list))
        FileOut_handle.write('LEGEND_LABELS\t%s\n' % '\t'.join(group_list))

        # write out data info
        FileOut_handle.write('\n# provide data here\nDATA\n')
        for leaf in Leaf_to_Group_dict:
            leaf_group = Leaf_to_Group_dict[leaf]
            leaf_color = Group_to_Color_dict[leaf_group]

            if ColorStrip is True:
                FileOut_handle.write('%s\t%s\t%s\n' % (leaf, leaf_color, leaf_group))
            if ColorRange is True:
                FileOut_handle.write('%s\trange\t%s\t%s\n' % (leaf, leaf_color, leaf_group))

        FileOut_handle.close()

    ####################################################################################################################

    # Prepare SimpleBar file
    if SimpleBar is True:

        if scale_str is None:
            print('Please provide scale values for barchart, in format 0-3-6-9')
            exit()

        # read in leaf value into dict
        leaf_value_dict = {}
        max_value = None
        for leaf_value in open(LeafValue):
            leaf_value_split = leaf_value.strip().split('\t')
            leaf_value_dict[leaf_value_split[0]] = float(leaf_value_split[1])

            # get max value
            if max_value == None:
                max_value = float(leaf_value_split[1])
            else:
                if float(leaf_value_split[1]) > max_value:
                    max_value = float(leaf_value_split[1])

        SimpleBar_FileOut_handle = open(FileOut, 'w')

        # write out header
        SimpleBar_FileOut_handle.write('DATASET_SIMPLEBAR\n')
        SimpleBar_FileOut_handle.write('# Reference: https://itol.embl.de/help/dataset_simplebar_template.txt\n')
        SimpleBar_FileOut_handle.write('\nSEPARATOR TAB\n')

        # write out SimpleBar attributes
        SimpleBar_FileOut_handle.write('\n# customize barchart attributes here\n')
        SimpleBar_FileOut_handle.write('DATASET_LABEL\t%s\n'    % LegendTitle)
        SimpleBar_FileOut_handle.write('COLOR\t%s\n'            % SimpleBar_COLOR)
        SimpleBar_FileOut_handle.write('WIDTH\t%s\n'            % SimpleBar_WIDTH)
        SimpleBar_FileOut_handle.write('MARGIN\t%s\n'           % MARGIN)
        SimpleBar_FileOut_handle.write('HEIGHT_FACTOR\t%s\n'    % SimpleBar_HEIGHT_FACTOR)
        SimpleBar_FileOut_handle.write('BORDER_WIDTH\t%s\n'     % SimpleBar_BORDER_WIDTH)

        # write out scale attributes
        scale_attributes_list = []
        for scale_value in scale_str.split('-'):
            scale_attributes = '%s-%s-%s-%s-%s-%s' % (scale_value, scale_value, SimpleBar_SCALE_COLOR, SimpleBar_SCALE_WIDTH, SimpleBar_SCALE_DASHED, SimpleBar_SCALE_FontSize)
            scale_attributes_list.append(scale_attributes)

        SimpleBar_FileOut_handle.write('\n# customize scale attributes here\n')
        SimpleBar_FileOut_handle.write('# format: VALUE-LABEL-COLOR-WIDTH-DASHED-LABEL_SCALE_FACTOR, LABEL_SCALE_FACTOR controls font size\n')
        SimpleBar_FileOut_handle.write('DATASET_SCALE\t%s\n' % '\t'.join(scale_attributes_list))

        # write out data info
        SimpleBar_FileOut_handle.write('\n# provide data here\n')
        SimpleBar_FileOut_handle.write('DATA\n')

        for leaf in leaf_value_dict:
            SimpleBar_FileOut_handle.write('%s\t%s\n' % (leaf, leaf_value_dict[leaf]))

        SimpleBar_FileOut_handle.close()

    ####################################################################################################################

    # Prepare Heatmap file
    if Heatmap is True:

        # read in leaf matrix into dict
        n = 0
        col_name_list = []
        leaf_matrix_dict = {}
        for leaf_matrix in open(LeafMatrix):
            leaf_matrix_split = leaf_matrix.strip().split('\t')
            if n == 0:
                col_name_list = leaf_matrix_split[1:]
            else:
                leaf_matrix_dict[leaf_matrix_split[0]] = leaf_matrix_split[1:]
            n += 1

        Heatmap_FileOut_handle = open(FileOut, 'w')

        # write out header
        Heatmap_FileOut_handle.write('DATASET_HEATMAP\n')
        Heatmap_FileOut_handle.write('# Reference https://itol.embl.de/help/dataset_heatmap_template.txt\n')
        Heatmap_FileOut_handle.write('\nSEPARATOR TAB\n')

        # write out heatmap attributes
        Heatmap_FileOut_handle.write('\n# customize heatmap attributes here\n')
        Heatmap_FileOut_handle.write('MARGIN\t%s\n'          % MARGIN)
        Heatmap_FileOut_handle.write('STRIP_WIDTH\t%s\n'     % Heatmap_STRIP_WIDTH)

        # write out legend info
        Heatmap_FileOut_handle.write('\n# customize legend here\n')
        Heatmap_FileOut_handle.write('AUTO_LEGEND\t1\n')
        Heatmap_FileOut_handle.write('DATASET_LABEL\t%s\n' % LegendTitle)
        Heatmap_FileOut_handle.write('USE_MID_COLOR\t1\n')
        Heatmap_FileOut_handle.write('COLOR_MIN\t#2980B9\n')
        Heatmap_FileOut_handle.write('COLOR_MID\t#ECF0F1\n')
        Heatmap_FileOut_handle.write('COLOR_MAX\t#E74C3C\n')
        Heatmap_FileOut_handle.write('\n# customize value range here. By default, color gradients will be calculated based on dataset values\n')
        Heatmap_FileOut_handle.write('# USER_MIN_VALUE	0\n')
        Heatmap_FileOut_handle.write('# USER_MID_VALUE	5\n')
        Heatmap_FileOut_handle.write('# USER_MAX_VALUE	10\n')
        Heatmap_FileOut_handle.write('\n# customize column name here\n')
        Heatmap_FileOut_handle.write('FIELD_LABELS\t%s\n' % '\t'.join(col_name_list))

        # write out data info
        Heatmap_FileOut_handle.write('\n# Provide data here\n')
        Heatmap_FileOut_handle.write('DATA\n')
        for leaf in leaf_matrix_dict:
            Heatmap_FileOut_handle.write('%s\t%s\n' % (leaf, '\t'.join(leaf_matrix_dict[leaf])))

        Heatmap_FileOut_handle.close()

    ####################################################################################################################

    # Prepare ExternalShape file
    if ExternalShape is True:

        # read in leaf matrix into dict
        n = 0
        col_name_list = []
        leaf_matrix_dict = {}
        for leaf_matrix in open(LeafMatrix):
            leaf_matrix_split = leaf_matrix.strip().split('\t')
            if n == 0:
                col_name_list = leaf_matrix_split[1:]
            else:
                leaf_matrix_dict[leaf_matrix_split[0]] = leaf_matrix_split[1:]
            n += 1

        Column_to_Color_dict = {}
        if ColumnColor is not None:
            for each_col in open(ColumnColor):
                each_col_split = each_col.strip().split('\t')
                Column_to_Color_dict[each_col_split[0]] = each_col_split[1]

        # check if all columns in data matrix are in Column_to_Color_dict
        if ColumnColor is not None:
            unfound_cols = []
            for each_col_header in col_name_list:
                if each_col_header not in Column_to_Color_dict:
                    unfound_cols.append(each_col_header)
            if len(unfound_cols) > 0:
                print('Color code for the following columns are not provided, program exited!')
                print(','.join(unfound_cols))
                exit()

        ExternalShape_FileOut_handle = open(FileOut, 'w')

        # write out header
        ExternalShape_FileOut_handle.write('DATASET_EXTERNALSHAPE\n')
        ExternalShape_FileOut_handle.write('# Reference https://itol.embl.de/help/dataset_external_shapes_template.txt\n')
        ExternalShape_FileOut_handle.write('\nSEPARATOR TAB\n')

        # define scale here
        if scale_str is None:
            print('Please provide scale values for ExternalShapes, e.g., 25-50-75-100, 2-4-6-8-10')
            exit()

        scale_list = scale_str.split('-')
        LEGEND_SHAPES_list = ['2'] * len(scale_list)
        LEGEND_COLORS_list = ['grey'] * len(scale_list)
        SHAPE_SCALES_list = scale_str_to_size_list(scale_str)

        ExternalShape_FileOut_handle.write('LEGEND_TITLE\t%s\n' % LegendTitle)
        ExternalShape_FileOut_handle.write('LEGEND_SHAPES\t%s\n' % '\t'.join(LEGEND_SHAPES_list))
        ExternalShape_FileOut_handle.write('LEGEND_COLORS\t%s\n' % '\t'.join(LEGEND_COLORS_list))
        ExternalShape_FileOut_handle.write('LEGEND_LABELS\t%s\n' % '\t'.join(scale_list))
        ExternalShape_FileOut_handle.write('LEGEND_SHAPE_SCALES\t%s\n' % '\t'.join(str(i) for i in SHAPE_SCALES_list))

        # write out ExternalShape attributes
        ExternalShape_FileOut_handle.write('\n# customize attributes here\n')
        ExternalShape_FileOut_handle.write('VERTICAL_GRID\t0\n')
        ExternalShape_FileOut_handle.write('HORIZONTAL_GRID\t0\n')
        ExternalShape_FileOut_handle.write('SHAPE_TYPE\t2\n')
        ExternalShape_FileOut_handle.write('COLOR_FILL\t1\n')
        ExternalShape_FileOut_handle.write('SHAPE_SPACING\t1\n')
        ExternalShape_FileOut_handle.write('SHOW_LABELS\t0\n')
        ExternalShape_FileOut_handle.write('SHOW_VALUES\t0\n')
        ExternalShape_FileOut_handle.write('DASHED_LINES\t0\n')

        if LegendTitle is None:
            ExternalShape_FileOut_handle.write('DATASET_LABEL\tExternalShape\n')
        else:
            ExternalShape_FileOut_handle.write('DATASET_LABEL\t%s\n' % LegendTitle)

        # write column header/color information
        ExternalShape_FileOut_handle.write('\n# customize column header/color here\n')
        ExternalShape_FileOut_handle.write('FIELD_LABELS\t%s\n' % '\t'.join(col_name_list))

        color_list = get_color_list(len(col_name_list))
        random.shuffle(color_list)
        if ColumnColor is not None:
            color_list = [Column_to_Color_dict[i] for i in col_name_list]
        ExternalShape_FileOut_handle.write('FIELD_COLORS\t%s\n' % '\t'.join(color_list))

        # write out data info
        ExternalShape_FileOut_handle.write('\n# Provide data here\n')
        ExternalShape_FileOut_handle.write('DATA\n')
        for leaf in leaf_matrix_dict:
            ExternalShape_FileOut_handle.write('%s\t%s\n' % (leaf, '\t'.join(leaf_matrix_dict[leaf])))

        ExternalShape_FileOut_handle.close()

    ####################################################################################################################


if __name__ == '__main__':

    # initialize the options parser
    parser = argparse.ArgumentParser()
    parser.add_argument('-ColorStrip',      required=False, action='store_true',   help='ColorStrip')
    parser.add_argument('-ColorRange',      required=False, action='store_true',   help='ColorRange')
    parser.add_argument('-SimpleBar',       required=False, action='store_true',   help='SimpleBar')
    parser.add_argument('-Heatmap',         required=False, action='store_true',   help='Heatmap')
    parser.add_argument('-ExternalShape',   required=False, action='store_true',   help='ExternalShape')
    parser.add_argument('-lg',              required=False, default=None,          help='Leaf Group')
    parser.add_argument('-gc',              required=False, default=None,          help='Specify Group/column Color (optional)')
    parser.add_argument('-cc',              required=False, default=None,          help='Specify Column Color (for ExternalShape format) (optional)')
    parser.add_argument('-lv',              required=False, default=None,          help='Leaf Value')
    parser.add_argument('-lm',              required=False, default=None,          help='Leaf Matrix')
    parser.add_argument('-scale',           required=False, default=None,          help='Scale Values, in format 0-3-6-9')
    parser.add_argument('-lt',              required=False, default=None,          help='Legend Title')
    parser.add_argument('-out',             required=True,                         help='Output filename')
    args = vars(parser.parse_args())
    iTOL(args)



import pysam


def get_reads_mapped_to_a_region(sam_file, seq_id, start_pos, end_pos):

    samfile = pysam.AlignmentFile(sam_file, "rb")
    iter = samfile.fetch(seq_id, start_pos, end_pos)

    mapped_read_id_list = []
    for mapped_read in iter:
        read_id = mapped_read.query_name
        mapped_read_id_list.append(read_id)

    return mapped_read_id_list


sam_file = '/Users/songweizhi/Desktop/pysam_test/ref_seq.bam'
mapped_read_list = get_reads_mapped_to_a_region(sam_file, 'ref_seq', 43, 44)
print(mapped_read_list)



#!/usr/bin/env python3

# Copyright (C) 2017, Weizhi Song
# songwz03@gmail.com

# get_reads_from_sam.py is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# get_reads_from_sam.py is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import argparse


def get_reads_from_sam(args):

    sam_file = args['sam']
    output_file = args['out']
    ctgs_file = args['ctgs']
    option = args['option']


    # get contig list
    ctg_list = []
    for each_ctg in open(ctgs_file):
        each_ctg = each_ctg.strip()
        ctg_list.append(each_ctg)


    # export reads
    output = open(output_file, 'w')
    for each in open(sam_file):
        if not each.startswith('@'):
            each_split = each.strip().split('\t')
            query_name = each_split[0]
            ref_name = each_split[2]
            query_seq = each_split[9]

            if option == 1:
                if ref_name in ctg_list:
                    output.write('>%s\n' % query_name)
                    output.write('%s\n' % query_seq)

            if option == 0:
                if ref_name not in ctg_list:
                    output.write('>%s\n' % query_name)
                    output.write('%s\n' % query_seq)

    output.close()


if __name__ == '__main__':

    parser = argparse.ArgumentParser()

    parser.add_argument('-sam', required=True, help='Input sam file')
    parser.add_argument('-ctgs', required=True, help='Contig list')
    parser.add_argument('-option', required=True, type=int, help="Specify '1' to get reads mapped to provided contigs, '0' to get reads not mapped to provided contigs")
    parser.add_argument('-out', required=True, help='Output fasta file')

    args = vars(parser.parse_args())

    get_reads_from_sam(args)



import os
import scrapy


wd  = '/Users/songweizhi/Desktop/scrapy_demo'

os.chdir(wd)



#os.system('scrapy startproject tutorial')


os.system('cd /Users/songweizhi/Desktop/scrapy_demo/tutorial')
os.system('scrapy crawl quotes')


# os.system(' ')




# os.system(' ')
# os.system(' ')
# os.system(' ')
# os.system(' ')
# os.system(' ')
# os.system(' ')
# os.system(' ')
# os.system(' ')
# os.system(' ')





__author__ = 'weizhisong'

#################################################################################
#                                                                               #
#             This script was writen to extract the information of              #
#          (a list of) interested taxon(s) from NCBI Taxonomy database          #
#                                                                               #
#################################################################################


file1 = open('/Users/weizhisong/Desktop/all_nodes_sorted_uniq.txt')
nodes_file = open('/Users/weizhisong/Research/NCBI_Taxonomy/taxdump/nodes.dmp')
names_file = open('/Users/weizhisong/Research/NCBI_Taxonomy/taxdump/names.dmp')
nodes_output = open('/Users/weizhisong/Desktop/customized_nodes.dmp', 'w')
names_output = open('/Users/weizhisong/Desktop/customized_names.dmp', 'w')


# Generate a list which contains all interested taxons
id_list = []

for file1_each_id in file1:
    file1_each_id = file1_each_id.rstrip('\n')
    id_list.append(file1_each_id)


# Extract information from name.dmp and export to a file named customized_names.dmp
for names_each_id in names_file:
    name_type = names_each_id.split('|')[3].rstrip('\t')[1:]
    names_each_id_id = names_each_id.split('|')[0].rstrip('\t')
    if name_type == 'scientific name':
        if names_each_id_id in id_list:
            names_output.write(names_each_id)
        else:
            pass
    else:
        pass

names_output.close()


# Extract information from nodes.dmp and export to a file named customized_nodes.dmp
for nodes_each_id in nodes_file:
    nodes_each_id_id = nodes_each_id.split('|')[0].rstrip('\t')
    if nodes_each_id_id in id_list:
        nodes_output.write(nodes_each_id)
    else:
        pass

nodes_output.close()



import re
import random
from bs4 import BeautifulSoup
from urllib.request import urlopen


base_url = 'https://baike.baidu.com/'
url = 'https://baike.baidu.com/item/%E7%BD%91%E7%BB%9C%E7%88%AC%E8%99%AB/5162711'


html = urlopen(url).read().decode('utf-8')
html_soup = BeautifulSoup(html, features='lxml')

print(html_soup.h1.get_text())
print(html_soup.find('h1').get_text())



import os
import argparse
from Bio import SeqIO
from Bio import AlignIO
from Bio.Seq import Seq
from Bio.SeqRecord import SeqRecord


sequence_manipulator_usage = '''
================================ Sequence manipulator example commands ===============================

BioSAK gbk2fa -gbk bin_1.gbk
BioSAK gbk2ffn -gbk bin_1.gbk
BioSAK gbk2faa -gbk bin_1.gbk
BioSAK ffn2faa -ffn bin_1.ffn

# get reverse complement sequence(s) for single and multiple sequence(s)
BioSAK get_rc -seq AAAAATTTTTGGGGGCCCCC
BioSAK get_rc -seq bin_1.ffn

======================================================================================================
'''


convert_align_format_usage = '''
================================ convert_align_format example commands ===============================

# convert alignment in fasta format to phylip-relaxed format
BioSAK convert_align_format -in NorthSea.aln -inf fasta -out NorthSea.phylip -outf phylip-relaxed

# Alignment format:
  clustal, emboss, fasta, fasta-m10, ig, maf, mauve, nexus, 
  phylip, phylip-sequential, phylip-relaxed, stockholm

# More details about alignment format is here: 
  https://biopython.org/wiki/AlignIO

======================================================================================================
'''


def sep_path_basename_ext(file_in):

    # separate path and file name
    file_path, file_name = os.path.split(file_in)
    if file_path == '':
        file_path = '.'

    # separate file basename and extension
    file_basename, file_extension = os.path.splitext(file_name)

    return file_path, file_basename, file_extension


def export_seq_record(gene_seq, gene_id, gene_description, seq_type, output_handle):

    seq_object = Seq(gene_seq)
    seq_record = SeqRecord(seq_object)
    seq_record.id = gene_id
    seq_record.description = gene_description
    SeqIO.write(seq_record, output_handle, 'fasta')


def gbk2fa(args):

    gbk_in = args['gbk']
    gbk_path, gbk_basename, gbk_extension = sep_path_basename_ext(gbk_in)
    ffn_out = '%s/%s.fa' % (gbk_path, gbk_basename)
    SeqIO.convert(gbk_in, 'genbank', ffn_out, 'fasta')


def gbk2ffn(args):

    gbk_in = args['gbk']
    gbk_path, gbk_basename, gbk_extension = sep_path_basename_ext(gbk_in)
    ffn_out = '%s/%s.ffn' % (gbk_path, gbk_basename)

    ffn_out_handle = open(ffn_out, 'w')
    for seq_record in SeqIO.parse(gbk_in, 'genbank'):
        for feature in seq_record.features:
            feature_locus_tag = feature.qualifiers['locus_tag'][0]

            if feature.location.strand == 1:
                feature_seq = str(seq_record.seq)[feature.location.start-1:feature.location.end]
            else:
                feature_seq_rc = str(seq_record.seq)[feature.location.start-1:feature.location.end]
                feature_seq = str(Seq(feature_seq_rc).reverse_complement())

            export_seq_record(feature_seq, feature_locus_tag, '', 'N', ffn_out_handle)

    ffn_out_handle.close()


def gbk2faa(args):

    gbk_in = args['gbk']
    gbk_path, gbk_basename, gbk_extension = sep_path_basename_ext(gbk_in)
    faa_out = '%s/%s.faa' % (gbk_path, gbk_basename)

    faa_out_handle = open(faa_out, 'w')
    for seq_record in SeqIO.parse(gbk_in, 'genbank'):
        for feature in seq_record.features:
            feature_locus_tag = feature.qualifiers['locus_tag'][0]
            feature_translation = feature.qualifiers['translation'][0]
            export_seq_record(feature_translation, feature_locus_tag, '', 'P', faa_out_handle)

    faa_out_handle.close()


def ffn2faa(args):

    ffn_in = args['ffn']
    ffn_path, ffn_basename, ffn_extension = sep_path_basename_ext(ffn_in)
    faa_out = '%s/%s.faa' % (ffn_path, ffn_basename)

    faa_out_handle = open(faa_out, 'w')
    for seq_record in SeqIO.parse(ffn_in, 'fasta'):
        seq_record_seq = seq_record.seq
        seq_record_seq_aa = seq_record_seq.translate()
        export_seq_record(str(seq_record_seq_aa), str(seq_record.id), '', 'P', faa_out_handle)
    faa_out_handle.close()


def get_rc(args):

    seq_in = args['seq']

    # check whether seq_in is a file
    if os.path.isfile(seq_in) is True:

        seq_in_path, seq_in_basename, seq_in_extension = sep_path_basename_ext(seq_in)
        seq_out = '%s/%s_rc%s' % (seq_in_path, seq_in_basename, seq_in_extension)

        seq_out_handle = open(seq_out, 'w')
        for seq_record in SeqIO.parse(seq_in, 'fasta'):
            seq_record_rc_id = '%s_rc' % seq_record.id
            seq_record_seq = seq_record.seq
            seq_record_seq_rc = seq_record_seq.reverse_complement()
            export_seq_record(str(seq_record_seq_rc), seq_record_rc_id, '', 'N', seq_out_handle)
        seq_out_handle.close()

    else:
        # check whether input is DNA sequence
        nc_bases = ['A', 'a', 'T', 't', 'G', 'g', 'C', 'c']
        nc_bases_count = 0
        for nc_base in nc_bases:
            nc_bases_count += seq_in.count(nc_base)

        if nc_bases_count == len(seq_in):
            seq_in_rc = str(Seq(seq_in).reverse_complement())
            print('>seq_in_rc\n%s\n' % seq_in_rc)

        else:
            print('Sequence file not exist or non DNA sequence detected, program exited!')
            exit()


def fq2fa(args):

    fq_in = args['fq']
    fq_path, fq_basename, fq_extension = sep_path_basename_ext(fq_in)
    fa_out = '%s/%s.fa' % (fq_path, fq_basename)

    SeqIO.convert(fq_in, 'fastq', fa_out, 'fastq')


def convert_align_format(args):

    aln_in =            args['in']
    aln_in_format =     args['inf']
    aln_out =           args['out']
    aln_out_format =    args['outf']

    AlignIO.convert(aln_in, aln_in_format, aln_out, aln_out_format)




import plotly

import plotly.plotly as py

data = dict(
    type='sankey',
    domain = dict(
      x =  [0,1],
      y =  [0,1]
    ),
    orientation = "h",
    valueformat = ".0f",
    valuesuffix = "TWh"
  )

layout =  dict(
    title = "Energy forecast for 2050<br>Source: Department of Energy & Climate Change, Tom Counsell via <a href='https://bost.ocks.org/mike/sankey/'>Mike Bostock</a>",
    font = dict(
      size = 10
    )
)


import plotly.plotly as py

import urllib, json

url = 'https://raw.githubusercontent.com/plotly/plotly.js/master/test/image/mocks/sankey_energy.json'
response = urllib.urlopen(url)
data = json.loads(response.read())

data_trace = dict(
    type='sankey',
    domain = dict(
      x =  [0,1],
      y =  [0,1]
    ),
    orientation = "h",
    valueformat = ".0f",
    valuesuffix = "TWh",
    node = dict(
      pad = 15,
      thickness = 15,
      line = dict(
        color = "black",
        width = 0.5
      ),
      label =  data['data'][0]['node']['label'],
      color =  data['data'][0]['node']['color']
    )
)

layout =  dict(
    title = "Energy forecast for 2050<br>Source: Department of Energy & Climate Change, Tom Counsell via <a href='https://bost.ocks.org/mike/sankey/'>Mike Bostock</a>",
    font = dict(
      size = 10
    )
)




import plotly.plotly as py

import urllib, json

url = 'https://raw.githubusercontent.com/plotly/plotly.js/master/test/image/mocks/sankey_energy.json'
response = urllib.urlopen(url)
data = json.loads(response.read())

data_trace = dict(
    type='sankey',
    width = 1118,
    height = 772,
    domain = dict(
      x =  [0,1],
      y =  [0,1]
    ),
    orientation = "h",
    valueformat = ".0f",
    valuesuffix = "TWh",
    node = dict(
      pad = 15,
      thickness = 15,
      line = dict(
        color = "black",
        width = 0.5
      ),
      label =  data['data'][0]['node']['label'],
      color =  data['data'][0]['node']['color']
    ),
    link = dict(
      source =  data['data'][0]['link']['source'],
      target =  data['data'][0]['link']['target'],
      value =  data['data'][0]['link']['value'],
      label =  data['data'][0]['link']['label']
  ))

layout =  dict(
    title = "Energy forecast for 2050<br>Source: Department of Energy & Climate Change, Tom Counsell via <a href='https://bost.ocks.org/mike/sankey/'>Mike Bostock</a>",
    font = dict(
      size = 10
    )
)

fig = dict(data=[data_trace], layout=layout)
py.iplot(fig, validate=False)












from requests import get
from bs4 import BeautifulSoup

MetaCyc_url = 'https://metacyc.org/META/class-instances?object=Reactions'
MetaCyc_url = 'https://metacyc.org/META/search-query?type=REACTION&name=1.1.1.-'

MetaCyc_soup = BeautifulSoup(get(MetaCyc_url).text, 'html.parser')

reaction_containers = MetaCyc_soup.find_all('tr', class_={'trDark', 'trLight'})

for reaction_container in reaction_containers:

    reaction = reaction_container.td.a.text
    reaction_id = reaction_container.td.a['href'].split('object=')[-1]

    print('%s\t%s' % (reaction_id, reaction))



import os
import glob


def sep_path_basename_ext(file_in):

    # separate path and file name
    file_path, file_name = os.path.split(file_in)
    if file_path == '':
        file_path = '.'

    # separate file basename and extension
    file_basename, file_extension = os.path.splitext(file_name)

    return file_path, file_basename, file_extension


MetaCyc_rxn_db_folder = '/Users/songweizhi/Desktop/MetaCyc_rxn_db'
combined_rxn_db = '/Users/songweizhi/Desktop/MetaCyc_rxn_db/MetaCyc_rxn_db.txt'


MetaCyc_rxn_db_file_re = '%s/*.dat' % MetaCyc_rxn_db_folder
MetaCyc_rxn_db_file_list = [os.path.basename(file_name) for file_name in glob.glob(MetaCyc_rxn_db_file_re)]


# get Attribute_list for all db files
Attribute_list = []
for MetaCyc_rxn_db_file in MetaCyc_rxn_db_file_list:

    pwd_MetaCyc_rxn_db_file = '%s/%s' % (MetaCyc_rxn_db_folder, MetaCyc_rxn_db_file)

    Attribute_line = False
    for rxn_attribute in open(pwd_MetaCyc_rxn_db_file):
        if rxn_attribute.startswith('#'):
            if '# Attributes:' in rxn_attribute:
                Attribute_line = True
            elif (Attribute_line is True) and (len(rxn_attribute) > 2):
                attribute_line_split = rxn_attribute.strip().split('    ')
                if attribute_line_split[1] not in Attribute_list:
                    Attribute_list.append(attribute_line_split[1])


combined_rxn_db_handle = open(combined_rxn_db, 'w')
combined_rxn_db_handle.write('Species\t%s\n' % '\t'.join(Attribute_list))
for MetaCyc_rxn_db_file in MetaCyc_rxn_db_file_list:

    pwd_MetaCyc_rxn_db_file = '%s/%s' % (MetaCyc_rxn_db_folder, MetaCyc_rxn_db_file)
    db_file_path, db_file_basename, db_file_extension = sep_path_basename_ext(pwd_MetaCyc_rxn_db_file)

    Attribute_line = False
    current_attribute_dict = {}
    for rxn in open(pwd_MetaCyc_rxn_db_file):
        if not rxn.startswith('#'):
            if rxn.strip() == '//':
                write_out_list = []
                for attribute in Attribute_list:
                    attribute_value = 'NA'
                    if attribute in current_attribute_dict:
                        attribute_value = '__'.join(current_attribute_dict[attribute])
                    write_out_list.append(attribute_value)
                combined_rxn_db_handle.write('%s\t%s\n' % (db_file_basename, '\t'.join(write_out_list)))
                current_attribute_dict = {}
            else:
                if ' - ' in rxn:
                    rxn_split = rxn.strip().split(' - ')
                    if rxn_split[0] not in current_attribute_dict:
                        current_attribute_dict[rxn_split[0]] = [rxn_split[1]]
                    else:
                        current_attribute_dict[rxn_split[0]].append(rxn_split[1])

combined_rxn_db_handle.close()




print(len(Attribute_list))



import numpy as np
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt


def box_plotter(num_list_in, label_list, output_plot):

    # turn num list into arrary
    MAG_HGT_num_lol_arrary = [np.array(i) for i in num_list_in]

    # get plot
    fig = plt.figure(1, figsize=(9, 6))
    ax = fig.add_subplot(111)
    bp = ax.boxplot(MAG_HGT_num_lol_arrary)

    # set x tick labels
    if len(label_list) <= 10:
        rotation_value = 0
    else:
        rotation_value = 270
    ax.set_xticklabels(label_list, rotation=rotation_value, fontsize=8)

    ## change the style of fliers and their fill
    for flier in bp['fliers']:
        flier.set(marker='+', color='black', alpha=0.7, markersize=3)

    plt.tight_layout()
    fig.savefig(output_plot, bbox_inches='tight', dpi=300)
    plt.close()


num_list_in = [[1,2,3,4,5], [4,5,6,7,8], [6,7,8,9,10]]
label_list = ['small', 'medium', 'high']
output_plot = '/Users/songweizhi/Desktop/test.png'



box_plotter(num_list_in, label_list, output_plot)






import numpy as np
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt


########################################################################################################################

def get_bar_plot(num_list_1, num_list_2, label_list, label_rotation, output_plot):

    # Set position of bar on X axis
    barWidth = 0.4
    r1 = np.arange(len(num_list_1))
    r2 = [x + barWidth for x in r1]

    plt.bar(r1, num_list_1, color='orange', width=barWidth, edgecolor='white', label='Number')
    plt.bar(r2, num_list_2, color='lightgreen', width=barWidth, edgecolor='white', label='Size')
    plt.xticks([r + barWidth for r in range(len(num_list_1))], label_list, rotation=label_rotation, fontsize=10)
    plt.title('The number and total size of bins')
    plt.legend(fontsize=10)
    plt.tight_layout()
    plt.savefig(output_plot, dpi=300)
    plt.close()


num_list_1 = [65, 29, 5, 99, 126, 111, 8, 60]
num_list_2 = [247, 86, 10, 370, 454, 439, 17, 172]
label_list = ['Apl', 'Car', 'Cli', 'Cos', 'Irc', 'Rho', 'Sty', 'swt']
label_rotation = 0
output_plot = '/Users/songweizhi/Desktop/test.png'
get_bar_plot(num_list_1, num_list_2, label_list, label_rotation, output_plot)


########################################################################################################################

def get_boxplot(num_lol, label_list, label_rotation, output_plot):

    num_lol_arrary = [np.array(i) for i in num_lol]
    fig = plt.figure(1, figsize=(9, 6))
    ax = fig.add_subplot(111)
    bp = ax.boxplot(num_lol_arrary)
    ax.set_xticklabels(label_list, rotation=label_rotation, fontsize=8)

    ## change the style of fliers and their fill
    for flier in bp['fliers']:
        flier.set(marker='+', color='black', alpha=0.7, markersize=3)

    plt.tight_layout()
    fig.savefig(output_plot, bbox_inches='tight', dpi=300)
    plt.close()


num_lol = [[1,2,3], [4,5,6], [7,8,9]]
label_list = ['Apl', 'Car', 'Cli']
label_rotation = 0
output_plot = '/Users/songweizhi/Desktop/boxplot.png'
get_boxplot(num_lol, label_list, label_rotation, output_plot)


########################################################################################################################

def get_categorical_scatter_plot(num_list, label_list, label_rotation, output_plot):

    num_index = list(range(1, len(num_list) + 1))

    plt.scatter(num_index, num_list)
    plt.xticks(num_index, label_list, rotation=label_rotation)
    plt.margins(0.2)
    plt.subplots_adjust(bottom=0.15)
    plt.tight_layout()
    plt.savefig(output_plot, dpi=300)
    plt.close()


num_list = [1, 2, 4, 3]
label_list = ['Frogs', 'Hogs', 'Bogs', 'Slogs']
get_categorical_scatter_plot(num_list, label_list, 0, '/Users/songweizhi/Desktop/categorical_scatter_plot.png')


# # add vertical lines
# for x_16s in ribosomal_RNA_16S_pos_list_dict['2.10_chromosome']:
#     plt.axvline(x=x_16s)


# # add horizontal lines
# for y_16s in ribosomal_RNA_16S_pos_list_dict['D2_c']:
#     plt.hlines(y=y_16s, xmin=0, xmax=ref_length_dict['2.10_chromosome'])




########## use * to grab excess items ##########

a1, b1, *rest1 = [1, 2, 3, 4, 5]
a2, *rest2, b2 = [1, 2, 3, 4, 5]
a3, *rest3, b3 = [1, 2, 3]
a4, *rest4, b4 = [1, 2]

print(rest1)    # [3, 4, 5]
print(rest2)    # [2, 3, 4]
print(rest3)    # [2]
print(rest4)    # []


########## swapping the values of variables without using a temporary variable ##########

a = 'apple'
b = 'orange'
a, b = b, a

print(a)    # orange
print(b)    # apple



